import mysql.connector
import datetime

class MySQLAbstraction:
    def __init__(self, username, password, host):
        self.username = username
        self.password = password
        self.host = host
    
    def connect(self):
        try:
            self.cnx = mysql.connector.connect(user=self.username, \
                password=self.password, host=self.host)
            #print('Connection established')
        except:
            print("Cannot establish connection")
            raise

    def disconnect(self):
        if self.cnx:
            self.cnx.close()
            #print('Connection closed')
        else:
            print('No connection to close!!')
            raise

    def createDB(self, dbName):
        self.cursor.execute("CREATE DATABASE %s" % (dbName))
    
    def dropDB(self, dbName):
        try:
            self.cursor.execute("DROP DATABASE %s" % (dbName))
        except:
            print("error dropping " + dbName)

    def getCursor(self):
        self.cursor = self.cnx.cursor(dictionary=True)

    def selectDB(self, dbName):
        try:
            self.cnx.database = dbName
        except:
            print("Cannot select DB: ")
            raise
    
    # this is useful if you're only manipulating one tableName
    # don't need to send table name again and again
    def setTableName(self, tableName):
        self.tableName = tableName

    def createTableDecat(self, tableName, numAttrs, numRows):
        #create table
        attrsList = ["a" + str(i) + " INT NOT NULL DEFAULT '0'" for i in range(numAttrs)]
        attrStr = ", ".join(attrsList) 
        query = "CREATE TABLE %s " % (tableName)
        query += "(id INT NOT NULL PRIMARY KEY, " + attrStr
        query += ", isSubject BOOL NOT NULL" + ")"
        self.cursor.execute(query)
        self.commit()

        #insert rows
        for i in range(1, int(numRows/2) + 1):
            self.cursor.execute("INSERT INTO %s (id, isSubject) VALUES ('%s', '1')" % (tableName, i))
        for i in range(int(numRows/2) + 1, numRows + 1):
            self.cursor.execute("INSERT INTO %s (id, isSubject) VALUES ('%s', '0')" % (tableName, i))
        self.commit()

    def createTableMVCCTSO(self, tableName, numAttrs, numRows):
        #create table
        attrsList = ["a" + str(i) + " INT NOT NULL DEFAULT '0'" for i in range(numAttrs)]
        attrStr = ", ".join(attrsList) 
        query = "CREATE TABLE %s " % (tableName)
        query += "(id INT NOT NULL, " + attrStr + ", "
        query += "wts DATETIME(6) NOT NULL, "
        query += "isSubject BOOL NOT NULL, "
        query +=  "PRIMARY KEY (id, wts))"
        
        self.cursor.execute(query)
        self.commit()
        
        #insert rows
        for i in range(1, int(numRows/2) + 1):
            self.cursor.execute("INSERT INTO %s (id, wts, isSubject) VALUES ('%s', '%s', '1')" % (tableName, i, datetime.datetime.now()))
        for i in range(int(numRows/2) + 1, numRows + 1):
            self.cursor.execute("INSERT INTO %s (id, wts, isSubject) VALUES ('%s', '%s', '0')" % (tableName, i, datetime.datetime.now()))
        self.commit()
        
    def createCommaSeparatedString(self, items):
        return ",".join(items)

    def selectNormal(self, id, attrs):
        attrsStr = self.createCommaSeparatedString(attrs)
        self.cursor.execute("SELECT %s FROM %s WHERE id='%s' LIMIT 1" % (attrsStr, self.tableName, id))
        return self.cursor.fetchone()

    def selectWithWts(self, id, wts, attrs):
        attrsStr = self.createCommaSeparatedString(attrs)
        self.cursor.execute("SELECT %s FROM %s WHERE id='%s' AND wts<='%s' ORDER BY wts DESC LIMIT 1" \
                    % (attrsStr, self.tableName, id, wts))
        return self.cursor.fetchone()

    def selectLatestVersion(self, id, attrs):
        attrsStr = self.createCommaSeparatedString(attrs)
        self.cursor.execute("SELECT %s FROM %s WHERE id='%s' ORDER BY wts DESC LIMIT 1" \
                    % (attrsStr, self.tableName, id))
        return self.cursor.fetchone()

    def updateNormal(self, id, attrs):

        #tmp _____________________________
        original = self.selectNormal(id, attrs)
        
        ret  = True
        what = {}
        #print("Attrs" + str(attrs))
        
        for attr in original:
            if attrs[attr] <= original[attr]:
                #print(str(id), str(attr), str(original[attr]), str(attrs[attr]))
                ret = False

        # _____________________________

        query = "UPDATE %s SET " % (self.tableName)
        for key, value in attrs.items():
            query += "%s='%s', " % (key, value)
        query = query[:-2]
        query += " WHERE id='%s'" % (id)
        self.cursor.execute(query)

        return ret

    # this actually creates a new row with the new timestamp
    # based on multi-versioning
    def updateWithWts(self, id, attr, newVal, wts):
        # get latest values
        allAttrs = ['a' + str(i) for i in range(0, 10)]
        allAttrs += ['isSubject']
        latestValues = self.selectLatestVersion(id, allAttrs)
        
        # use that as a base to create a new row (version)
        for k in latestValues:
            if k == attr:
                latestValues[k] = newVal

        query = "INSERT INTO %s " % (self.tableName)
        query += "(id, wts, " + self.createCommaSeparatedString(allAttrs) + ")"
        query += " VALUES ('%s', '%s'" % (id, wts)

        for a in allAttrs:
            query += ", '%s'" % (latestValues[a])

        query += ")"

        #print(query)
        self.cursor.execute(query)

    # do this manually if you have to 
    # by default autocommit is turned off
    def commit(self):
        self.cnx.commit()


# testing
'''
db = MySQLAbstraction('root', 'access_control_2017', '127.0.0.1')
db.connect()
db.getCursor()

db.dropDB('testing123')

db.createDB('testing123')
db.commit()
db.selectDB('testing123')

db.createTableDecat('data', 10, 10)
db.commit()

db.setTableName('data')
d = db.selectNormal(1, ['a0', 'a1', 'a3'])

db.disconnect()
'''