import json
import sys
import matplotlib.pyplot as plt
from Filenames import *
import os
import pickle

DPI = 500

def createFolderIfNotExists(folderName):
    directory = os.path.dirname(folderName)
    if not os.path.exists(directory):
        os.makedirs(directory)

def plotToFile(xLabel, xData, yLabel, yData, title, filename):
    plt.plot(xData, yData)
    plt.title(title)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.savefig(filename, dpi=DPI)
    plt.close()

def plotQuantity(title, xLabel, yLabel, plotTofilename, configs, funcToGetXDataFromConfig, funcToGetYDataFromStats):
    xData = []
    yData = []
    for config in configs:
        x = funcToGetXDataFromConfig(config)
        f = open(getPickleFilename(config), "rb")
        stats = pickle.load(f)
        f.close()
        y = funcToGetYDataFromStats(stats)
        xData.append(x)
        yData.append(y)

    plotToFile(xLabel, xData, yLabel, yData, title, plotTofilename)


def print_usage():
    print("Usage: python3 Plotter <config>")

def main():
    if len(sys.argv) != 2:
        print_usage()
        exit()
    configFile = open(sys.argv[1])
    configFile = json.load(configFile)
    quantityBeingChanged = list(configFile['experiments'][0])[0]

    configs = []
    for config in configFile['experiments']:
        configs.append(config)
    for config in configs:
        for attr in configFile:
            if not attr == "experiments":
                config[attr] = configFile[attr]
    
    createFolderIfNotExists("./graphs")
    createFolderIfNotExists("graphs/" + configFile['experimentName'] + "/")
    

    plotQuantity("Average Throughput", quantityBeingChanged, "Throughput (requests per second)", \
        getThroughputGraphFilename(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['aggregate']['throughput'])
    
    plotQuantity("Average Latency", quantityBeingChanged, "Latency (in seconds)", \
        getLatencyGraphFilename(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['requests']['latencyStats']['mean'])
    
    plotQuantity("fConflict" + quantityBeingChanged, quantityBeingChanged, "fConflict", \
        getfConflictGraphFilename(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['requests']['fConflict'])
    
    plotQuantity("Average Network Messages", quantityBeingChanged, "Network messages per request", \
        getNetworkMessagePerRequestGraphFilename(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['aggregate']['networkMessagesPerRequest'])
    
    plotQuantity("Average Total Messages", quantityBeingChanged, "Messages per request", \
        getMessagePerRequestGraphFilename(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['aggregate']['messagesPerRequest'])

main()

'''
x = [i for i in range(0, 11)]
y = [i*i for i in x]
plotToFile("x", x, "x*x", y, "Squared", "squared.png")
'''