import datetime
import sys

MAX_DBLATENCY = 1 * (10**6) # in microseconds
SECONDS_IN_DAY = 86400

def canEvict(version):
    delta = datetime.datetime.now() - version.ts
    deltaInSeconds = (delta.days * SECONDS_IN_DAY) + delta.seconds
    deltaInMicroSeconds = deltaInSeconds*(10**6) + delta.microseconds
    return deltaInMicroSeconds > MAX_DBLATENCY

class Version:
    def __init__(self, id, attr, val, ts):
        self.id = id
        self.attr = attr
        self.val = val
        self.ts = ts
    
    def __str__(self):
        return "(" + str(self.id) + ", " + str(self.attr) + ") -> (" + str(self.val) + ", " + str(self.ts) + ")"

class Node:
    def __init__(self, value, prev=None, next=None):
        self.value = value
        self.prev = prev
        self.next = next

class LRUCache:
    dummyValue = -1

    def __init__(self, maxSize, canEvictFunc=canEvict):
        self.maxSize = maxSize
        self.size = 0
        self.dict = {}
        self.canEvictFunc = canEvictFunc

        # head and tail always point to dummy nodes
        self.head = Node(self.dummyValue)
        self.tail = Node(self.dummyValue)
        self.head.next = self.tail
        self.tail.prev = self.head

    def full(self):
        return self.size  == self.maxSize


    def debug(self):
        node = self.head.next
        while node != self.tail:
            print(node.value)
            print()
            node = node.next

    # returns Version object defined at the top of this file
    def getLatestVersionBefore(self, id, attr, ts=None):
        if ts == None:
            ts = datetime.datetime.now()

        if id in self.dict and attr in self.dict[id]:
            for node in self.dict[id][attr]:
                version = node.value
                if version.ts < ts:
                    return version
        return None

    def getCachedUpdatesBefore(self, id, ts=None):
        if ts == None:
            ts = datetime.datetime.now()
        
        if id not in self.dict:
            return None
        cachedUpdates = {}

        for attr in self.dict[id]:
            for node in self.dict[id][attr]:
                version = node.value
                if version.ts < ts:
                    cachedUpdates[attr] = version
                    break
        
        return cachedUpdates
        
    # returns Version object
    def updateVersion(self, id, attr, val, ts=None):
        
        # _____________tmp_______________
        if ts == None:
            ts = datetime.datetime.now()
        #__________________________________

        if self.full():
            if not self.evict():
                print("Unable to evict entry from LRUCache: try increasing maxSize?")
                return None
        
        # create a node with new version, and insert it at the head of the list
        node = Node(Version(id, attr, val, ts), self.head, self.head.next)
        self.head.next.prev = node
        self.head.next = node
        self.size += 1
        
        # add it to dict
        if id not in self.dict:
            self.dict[id] = {}
        if attr not in self.dict[id]:
            self.dict[id][attr] = []
        
        listOfNodes = self.dict[id][attr]
        i = 0
        while i < len(listOfNodes):
            if listOfNodes[i].value.ts < ts:
                break
            i += 1
        listOfNodes.insert(i, node)
        return node.value

    # starting from tail 
    # go through the list until canEvictFunc says True
    def evict(self):
        node = self.tail.prev
        while node != self.head:
            if self.canEvictFunc(node.value):
                # remove it from dict list of versions
                version = node.value
                for n in self.dict[version.id][version.attr]:
                    if n == node:
                        self.dict[version.id][version.attr].remove(n)
                        break
                
                # remove it from linked list
                node.prev.next = node.next
                node.next.prev = node.prev
                self.size -= 1
                return True
            node = node.prev
        return False
    
    def debug(self):
        for id in self.dict:
            print("ID: " + str(id))
            for attr in self.dict[id]:
                print("attr: " + str(attr)),
                for node in self.dict[id][attr]:
                    print(node.value)
                print()