import json
import sys
import matplotlib.pyplot as plt
from Filenames import *
import os
import pickle
import copy

DPI = 500

def createFolderIfNotExists(folderName):
    directory = os.path.dirname(folderName)
    if not os.path.exists(directory):
        os.makedirs(directory)

def plotToFile(xLabel, xData, yLabel, yData, legend, linestyle, color, title, filename):
    for i in range(len(xData)):
        plt.plot(xData[i], yData[i], linestyle=linestyle[i], color=color[i], label=legend[i])
    plt.title(title)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.legend()
    plt.savefig(filename, dpi=DPI)
    plt.close()
 
'''
#testing
xData = [   [ [0,1,2,3,4,5], [0,1,2,3,4,5] ],  [ [0,1,2,3,4,5], [0,1,2,3,4,5]  ]  ]
yData = [   [ [], [] ],  [ [], []  ]  ]

yData[0][0] = [x for x in xData[0][0]]
yData[0][1] = [2*x for x in xData[0][1]]
yData[1][0] = [x*x for x in xData[1][0]]
yData[1][1] = [x*x*x for x in xData[1][1]]
legend = [   [ "wrongWrite=0; wrongAttr=0", "wrongWrite=0; wrongAttr=1" ],  [ "wrongWrite=1; wrongAttr=0", "wrongWrite=1; wrongAttr=1"  ]  ]
linestyle = [   [ "solid", "solid" ],  [ "dashed", "dashed"  ]  ]
color = [   [ "#550000", "#005500" ],  [ "#000055", "#222200"  ]  ]

plotToFile("x", xData, "val", yData, legend, linestyle,  color, "throughput", "test.png")
'''

def getBaseExperimentNameDecat(config):
    return "nObj-" + str(config['nObj']) + "_nReq-" + str(config['nRequest']) + "_nClient-" + str(config['nClient']) + \
             "_nWorker-" + str(config['nWorker']) + "_pWrite-" + str(config['pWrite']) + "_pSameCoord-" + str(config['pSameCoord'])

def getPickleFilenameForDecat(config):
    return "../Decat merged version/" + config['statsFolder'] + "/" + config['experimentName'] + "/" + getBaseExperimentNameDecat(config) + ".pickle"

def plotQuantity(title, xLabel, yLabel, plotTofilename, configs, funcToGetXDataFromConfig, funcToGetYDataFromStats):
    xData = [[], [], [], [], []]
    yData = [[], [], [], [], []]
    legend = ["Decat et al's algorithm", "wrongWrite=0; wrongAttr=0", "wrongWrite=0; wrongAttr=1", "wrongWrite=1; wrongAttr=0", "wrongWrite=1; wrongAttr=1"]
    linestyle = ["dotted", "solid", "solid", "dashdot", "dashed"]
    color = ["#d91f1f", "#53de21", "#3424e3", "#e6d327", "#eb2ae1"]

    for config_ in configs:
        i = 0
        config = copy.deepcopy(config_)
        configDecat = copy.deepcopy(config_)
        x = funcToGetXDataFromConfig(config)
        f = open(getPickleFilenameForDecat(config), "rb")
        stats = pickle.load(f)
        f.close()
        y = funcToGetYDataFromStats(stats)
        xData[i].append(x)
        yData[i].append(y)

        for pair in [(0,0), (0,1), (1,0), (1,1)]:
            i += 1
            wrongWrite = pair[0]
            wrongAttr = pair[1]
            config['wrongWrite'] = wrongWrite
            config['wrongAttr'] = wrongAttr
            
            x = funcToGetXDataFromConfig(config)
            f = open(getPickleFilename(config), "rb")
            stats = pickle.load(f)
            f.close()
            y = funcToGetYDataFromStats(stats)
            xData[i].append(x)
            yData[i].append(y)
    #print(xData)
    #print(yData)
    plotToFile(xLabel, xData, yLabel, yData, legend, linestyle, color, title, plotTofilename)


def print_usage():
    print("Usage: python3 Plotter <config>")

def main():
    if len(sys.argv) != 2:
        print_usage()
        exit()
    
    configFile = open(sys.argv[1])
    configFile = json.load(configFile)

    quantityBeingChanged = list(configFile['experiments'][0])[0]

    configs = []
    for config in configFile['experiments']:
        configs.append(config)
    for config in configs:
        for attr in configFile:
            if not attr == "experiments":
                config[attr] = configFile[attr]
    
    createFolderIfNotExists("./graphs5")
    createFolderIfNotExists("graphs5/" + configFile['experimentName'] + "/")


    plotQuantity("Average Throughput", quantityBeingChanged, "Throughput (requests per second)", \
        getThroughputGraphFilename5(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['aggregate']['throughput'])
    
    plotQuantity("Average Latency", quantityBeingChanged, "Latency (in seconds)", \
        getLatencyGraphFilename5(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['requests']['latencyStats']['mean'])
    
    plotQuantity("fConflict", quantityBeingChanged, "fConflict", \
        getfConflictGraphFilename5(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['requests']['fConflict'])
    
    plotQuantity("Average Network Messages", quantityBeingChanged, "Network messages per request", \
        getNetworkMessagePerRequestGraphFilename5(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['aggregate']['networkMessagesPerRequest'])
    
    plotQuantity("Average Total Messages", quantityBeingChanged, "Messages per request", \
        getMessagePerRequestGraphFilename5(configFile), configs, lambda c : c[quantityBeingChanged], \
        lambda stats: stats['aggregate']['messagesPerRequest'])
    
main()