import random

class RequestGeneratorDecat:
    def __init__(self, seed, obj1Ids, obj2Ids, coord1Objects, coord2Objects, pWrite, pSameCoord):
        self.randomizer = random.Random()
        self.randomizer.seed(seed)

        # we partition all objects into either obj1 or obj2
        # the following are lists that contain the obj1Ids and obj2Ids
        self.obj1Ids = obj1Ids
        self.obj2Ids = obj2Ids
        
        # objects assigned to coord1
        self.coord1Objects = coord1Objects

        # objects assigned to coord2
        self.coord2Objects = coord2Objects
        
        self.pWrite = pWrite
        self.pSameCoord = pSameCoord
        
    def generateNext(self):
        request = {
            'obj1Id' : '',
            'obj2Id' : '',
            'action' : ''
        }
        
        if self.pSameCoord == -1:
            # choose both uniformly at random
            request['obj1Id'] = self.randomizer.choice(self.obj1Ids)
            request['obj2Id'] = self.randomizer.choice(self.obj2Ids)
        else:
            # we are explicitly controlling pSameCoord
            
            request['obj1Id'] = self.randomizer.choice(self.obj1Ids)
            sameCoord = self.trueWithProbability(self.pSameCoord)
            if sameCoord:
                # we'll look for the obj2Id in the same coord
                coord = self.coord1Objects if request['obj1Id'] in self.coord1Objects else self.coord2Objects
            else:
                # we'll look for the obj2Id in the other coord
                coord = self.coord2Objects if request['obj1Id'] in self.coord1Objects else self.coord1Objects
            
            # Note: this might end in an infinite loop if we can't the desired obj2 in the
            # right coord
            # This is not an issue with >50 objects though
            while True:
                    request['obj2Id'] = self.randomizer.choice(self.obj2Ids)
                    if request['obj2Id'] in coord:
                        break
            # at this point we have the unique ids of subject(obj1) and resource(obj2)
            # from either the same coordinator or different coordinators based on pSameCoord

        # now determine whether request is rd-only or write-req
        # and if write-req, whether it writes obj1 or obj2
        if self.trueWithProbability(self.pWrite):
            # it's a write-req. now choose among 1 or 2
            request['action'] += 'w' + str(self.randomizer.choice([1, 2]))
        else:
            request['action'] += 'rd'
        
        # if request is write-req, determine the index of written attr
        # note: hardcoded: we only allow the indices [0,1] to be written
        if request['action'] != 'rd':
            request['action'] += str(self.randomizer.choice([0,1]))
        else:
            request['action'] += '-'
        
        
        # determine index of first obj1(subject) attr read
        # note: hardcoded: the possible indices are 0..9
        request['action'] += str(self.randomizer.choice(range(0,10)))

        # read 1 or 2 more attrs
        numobj1MoreAttrsRead = self.randomizer.choice([1, 2])
        for i in range(1, 3):
            if numobj1MoreAttrsRead > 0:
                request['action'] += str(self.randomizer.choice(range(0, 10)))
            else:
                request['action'] += '-'
            numobj1MoreAttrsRead -= 1


        # determine index of first obj2(resource) attr read
        # note: hardcoded: the possible indices are 0..9
        request['action'] += str(self.randomizer.choice(range(0,10)))

        # read 1 or 2 more attrs
        numobj2MoreAttrsRead = self.randomizer.choice([1, 2])
        for i in range(1, 3):
            if numobj2MoreAttrsRead > 0:
                request['action'] += str(self.randomizer.choice(range(0, 10)))
            else:
                request['action'] += '-'
            numobj2MoreAttrsRead -= 1
        
        return request

    def trueWithProbability(self, p):
        return self.randomizer.random() < p



# let's test this
'''
r = RequestGeneratorDecat(23, range(1, 11), range(11, 21), \
        [1, 2, 3, 7, 8, 11, 14, 16, 17, 19], [4, 5, 6, 9, 10, 12, 13, 15, 18, 20], \
        0.5, 0)
for i in range(5):
    print(r.generateNext())
'''