from RequestGeneratorDecat import RequestGeneratorDecat
import random

class RequestGeneratorMVCCTSO:
    def __init__(self, seedForBaseDecat, seedForMVCCTSO, obj1Ids, obj2Ids, coord1Objects, coord2Objects, pWrite, pSameCoord, pWrongO, pWrongA):
         self.basicRequestGenerator = RequestGeneratorDecat(seedForBaseDecat, obj1Ids, obj2Ids, coord1Objects, coord2Objects, pWrite, pSameCoord)
         self.randomizer = random.Random()
         self.randomizer.seed(seedForMVCCTSO)
         self.pWrongO = pWrongO
         self.pWrongA = pWrongA
    
    def generateNext(self):
        # this is the basic request from Decat
        # we'll add more fields to it for MVCC-TSO
        request = self.basicRequestGenerator.generateNext()

        # add mightWriteObj attr
        request['mightWriteObj'] = []
        if request['action'][0:2] == 'rd':
            if self.trueWithProbability(self.pWrongO):
                request['mightWriteObj'].append(self.randomizer.choice([1, 2]))
        else:
            writtenObj = int(request['action'][1])
            request['mightWriteObj'].append(writtenObj)
            if self.trueWithProbability(self.pWrongO):
                otherObj = 1 if writtenObj == 2 else 2
                request['mightWriteObj'].append(otherObj)
                # note: this is needed so that the written object is not always at the
                # front of the list, which will never cause incorrect prediction
                if self.trueWithProbability(0.5): # half the time
                    # swap
                    request['mightWriteObj'][0], request['mightWriteObj'][1] = request['mightWriteObj'][1], request['mightWriteObj'][0]


        # add fields mightWriteAttr, mightReadAttr and defReadAttr
        # add values to them later based on pWrongA
        for index in ['mightWriteAttr', 'mightReadAttr', 'defReadAttr']:
            request[index] = {}
            for obj in [1, 2]:
                request[index][obj] = []


        # now start filling in actual values based on pWrongA
        if self.pWrongA == 0:
            # everything should be accurate
            # parse action and use it to add more fields to request that are correct
            # note: mightReadAttr will be empty since pWrongA == 0

            # fill in mightWriteAttr correctly
            if request['action'][0:2] != 'rd':
                writtenObj = int(request['action'][1])
                writtenAttr = 'a' + request['action'][2]
                request['mightWriteAttr'][writtenObj].append(writtenAttr)
            
            # fill in defReadAttr correctly
            for obj in [1, 2]:
                # the offsets below index into action
                # start from 3 for obj1, and from 6 for obj2
                attrOffsetStart = 3 if obj == 1 else 6
                attrOffsetEnd = attrOffsetStart + 3
                for attrRead in request['action'][attrOffsetStart:attrOffsetEnd]:
                    if attrRead != '-':
                        attr = 'a' + attrRead
                        request['defReadAttr'][obj].append(attr)
            

        elif self.pWrongA == 1:
            # maximally inaccurate
            # note: defReadAttr is empty since pWrongA == 1

            # mightWriteAttr contains both mutable attributes for the written object
            for obj in request['mightWriteObj']:
                request['mightWriteAttr'][obj].extend(['a0', 'a1'])
            
            # mightReadAttr contains all attrs for both obj1 and obj2
            for obj in [1, 2]:
                request['mightReadAttr'][obj].extend(['a' + str(i) for i in range(0, 10)])


        else:
            print("Invalid value for pWrongA")
            raise

        return request



    def trueWithProbability(self, p):
        return self.randomizer.random() < p

# testing
r = RequestGeneratorMVCCTSO(23, 24, range(1, 11), range(11, 21), \
        [1, 2, 3, 7, 8, 11, 14, 16, 17, 19], [4, 5, 6, 9, 10, 12, 13, 15, 18, 20], \
        0.5, 0, 1, 1)
for i in range(5):
    print(r.generateNext())
